import uuid
from sqlalchemy import orm

from viggocore.database import db
from viggocore.common.subsystem import entity


class CertificadoDigital(entity.Entity, db.Model):

    attributes = ['domain_org_id']
    attributes += entity.Entity.attributes

    domain_org_id = db.Column(
        db.CHAR(32), db.ForeignKey('domain_org.id'), nullable=False,
        unique=True)
    certificado = db.Column(db.Text, nullable=False)
    senha = db.Column(db.Text, nullable=False, default=uuid.uuid4().hex)

    domain_org = orm.relationship(
        'DomainOrg', backref=orm.backref('certificado_digital_domain_org'))

    def __init__(self, id, domain_org_id, certificado, senha,
                 active=True, created_at=None, created_by=None,
                 updated_at=None, updated_by=None, tag=None):
        super().__init__(id, active, created_at, created_by,
                         updated_at, updated_by, tag)
        self.domain_org_id = domain_org_id
        self.certificado = certificado
        self.senha = senha

    @classmethod
    def individual(cls):
        return 'certificado_digital'



from viggocore.common.subsystem import operation
from viggocore.common.subsystem.pagination import Pagination
from viggocore.common import manager

from viggofiscal.subsystem.parametrizacao.tipo_operacao.resource \
    import TipoOperacao


class Create(operation.Create):

    def pre(self, session, **kwargs):
        codigo = kwargs.get('codigo', None)
        domain_id = kwargs.get('domain_org_id', None)

        if not codigo and domain_id:
            kwargs['codigo'] = self.manager.api.domain_sequences().\
                get_nextval(id=domain_id, name=TipoOperacao.CODIGO_SEQUENCE)

        return super().pre(session=session, **kwargs)


class List(operation.List):

    def do(self, session, **kwargs):
        query = session.query(TipoOperacao)

        query = self.manager.apply_filters(query, TipoOperacao, **kwargs)
        query = query.distinct()

        total_rows = None
        if self.manager.with_pagination(**kwargs):
            total_rows = query.count()

        pagination = Pagination.get_pagination(TipoOperacao, **kwargs)
        if pagination.order_by is not None:
            pagination.adjust_order_by(TipoOperacao)
        query = self.driver.apply_pagination(query, pagination)
        result = query.all()

        return (result, total_rows)


class Manager(manager.CommonManager):

    def __init__(self, driver):
        super(Manager, self).__init__(driver)
        self.create = Create(self)
        self.list = List(self)



import base64
import hashlib

from cryptography.fernet import Fernet

from viggocore.common.subsystem import operation
from viggocore.common import manager, exception


class Create(operation.Create):

    def pre(self, session, file, **kwargs):
        self.file = file

        senha = kwargs.get('senha', None)
        fernet = Fernet(self.manager._get_key_hash())
        senha_hash = str(fernet.encrypt(senha.encode()))

        kwargs['senha'] = senha_hash
        kwargs['certificado'] = ''

        response_pre = super().pre(session=session, **kwargs)
        return response_pre

    def do(self, session, **kwargs):
        encoded_file = base64.b64encode(self.file.read())
        self.entity.certificado = str(encoded_file)

        super().do(session=session, **kwargs)
        return self.entity


class Update(operation.Update):

    def pre(self, session, id, file, **kwargs):
        super().pre(session=session, id=id, **kwargs)
        self.file = file

        senha = kwargs.get('senha', None)
        senha_anterior = kwargs.pop('senha_anterior', None)
        if senha and not senha_anterior:
            raise exception.BadRequest(
                'Para alterar a senha deve informar a senha anterior!')

        elif senha and senha_anterior:
            fernet = Fernet(self.manager._get_key_hash())
            senha_db = fernet.decrypt(self.entity.senha[2:-1].encode()).decode()
            if senha_db != senha_anterior:
                raise exception.BadRequest('A Senha anterior está errada!')

            senha_hash = str(fernet.encrypt(senha.encode()))
            self.senha = senha_hash

        return self.entity.is_stable()

    def do(self, session, **kwargs):
        kwargs.pop('file', None)
        kwargs.pop('senha_anterior', None)
        kwargs['senha'] = self.senha
        if self.file:
            encoded_file = base64.b64encode(self.file.read())
            self.entity.certificado = str(encoded_file)

        super().do(session=session, **kwargs)
        return self.entity


class Manager(manager.CommonManager):

    def __init__(self, driver):
        super(Manager, self).__init__(driver)
        self.create = Create(self)
        self.update = Update(self)

    def _get_key_hash(self):
        KEY = 'PBKDF2WithHmacSHA1'
        key_hash = hashlib.sha256(KEY.encode()).digest()
        return base64.urlsafe_b64encode(key_hash)



from sqlalchemy.sql.schema import UniqueConstraint

from viggocore.database import db
from viggocore.common.subsystem import entity


class NcmIbptErro(entity.Entity, db.Model):

    attributes = ['ncm', 'uf', 'chave', 'versao', 'tipo',
                  'filename', 'msg_erro', 'erro_em', 'extipi', 'corrigido']
    attributes += entity.Entity.attributes

    ncm = db.Column(db.String(20), nullable=False)
    uf = db.Column(db.CHAR(2), nullable=False)
    chave = db.Column(db.String(100), nullable=False)
    versao = db.Column(db.String(100), nullable=False)
    tipo = db.Column(db.Numeric(3), nullable=False)
    filename = db.Column(db.String(2000), nullable=False)
    msg_erro = db.Column(db.String(2000), nullable=False)
    erro_em = db.Column(db.DateTime(), nullable=False)
    extipi = db.Column(db.Numeric(), nullable=True)
    corrigido = db.Column(db.Boolean(), nullable=False,
                          default=False, server_default='false')

    __table_args__ = (
        UniqueConstraint(
            'ncm', 'uf', 'chave', 'versao', 'extipi', 'tipo',
            name='ncm_ibpt_erro_ncm_uf_chave_versao_extipi_tipo_uk'),)

    def __init__(self, id, ncm, uf, chave, versao, tipo, filename, msg_erro,
                 erro_em, extipi=None, corrigido=False,
                 active=True, created_at=None, created_by=None,
                 updated_at=None, updated_by=None, tag=None):
        super().__init__(id, active, created_at, created_by,
                         updated_at, updated_by, tag)
        self.ncm = ncm
        self.uf = uf
        self.chave = chave
        self.versao = versao
        self.tipo = tipo
        self.filename = filename
        self.msg_erro = msg_erro
        self.erro_em = erro_em
        self.extipi = extipi
        self.corrigido = corrigido

    @classmethod
    def individual(cls):
        return 'ncm_ibpt_erro'



import flask
import json

from viggocore.common import exception, utils
from viggocore.common.subsystem import controller


class Controller(controller.Controller):

    MIMETYPE_JSON = 'application/json'

    def __init__(self, manager, resource_wrap, collection_wrap):
        super().__init__(manager, resource_wrap, collection_wrap)

    def create(self):
        try:
            values = flask.request.values.get('body', None)
            file = flask.request.files.get('file', None)

            file_types = ['pfx', 'p12']
            if not file:
                raise exception.BadRequest(
                    'ERRO! Arquivo não enviado na requisição')

            elif file.filename.split('.')[-1] not in file_types:
                raise exception.BadRequest(
                    'O file não é de um tipo suportado (pfx ou p12).')

            data = json.loads(values)
            result = self.manager.create(file=file, **data)
            response = {self.resource_wrap: result.to_dict()}
        except exception.ViggoCoreException as exc:
            return flask.Response(response=exc.message,
                                  status=exc.status)

        return flask.Response(response=utils.to_json(response),
                              status=201,
                              mimetype=self.MIMETYPE_JSON)

    def update(self, id):
        try:
            values = flask.request.values.get('body', None)
            file = flask.request.files.get('file', None)

            if file:
                file_types = ['pfx', 'p12']
                if file.filename.split('.')[-1] not in file_types:
                    raise exception.BadRequest(
                        'O file não é de um tipo suportado (pfx ou p12).')

            data = json.loads(values)
            result = self.manager.update(file=file, **data)
            response = {self.resource_wrap: result.to_dict()}
        except exception.ViggoCoreException as exc:
            return flask.Response(response=exc.message,
                                  status=exc.status)

        return flask.Response(response=utils.to_json(response),
                              status=201,
                              mimetype=self.MIMETYPE_JSON)



from viggofiscal.subsystem.calculo_fiscal.icms.base_reduzida_icms_proprio \
    import BaseReduzidaIcmsProprio
from viggofiscal.subsystem.calculo_fiscal.icms.base_icms_proprio \
    import BaseIcmsProprio
from viggofiscal.subsystem.calculo_fiscal.icms.valor_icms_proprio \
    import ValorIcmsProprio
from viggofiscal.subsystem.calculo_fiscal.icms.base_icms_st \
    import BaseIcmsST
from viggofiscal.subsystem.calculo_fiscal.icms.base_reduzida_icms_st \
    import BaseReduzidaIcmsST
from viggofiscal.subsystem.calculo_fiscal.icms.valor_icms_st \
    import ValorIcmsST


class Icms202_203():

    def __init__(self, valor_produto: float, valor_frete: float,
                 valor_seguro: float, despesas_acessorias: float,
                 valor_desconto: float, aliq_icms_proprio: float,
                 aliq_icms_st: float, mva: float,
                 percentual_reducao: float,
                 percentual_reducao_st: float):
        self.valor_produto = float(valor_produto)
        self.valor_frete = float(valor_frete)
        self.valor_seguro = float(valor_seguro)
        self.despesas_acessorias = float(despesas_acessorias)
        self.valor_desconto = float(valor_desconto)
        self.aliq_icms_proprio = float(aliq_icms_proprio)
        self.aliq_icms_st = float(aliq_icms_st)
        self.mva = float(mva)
        self.percentual_reducao = float(percentual_reducao)
        self.percentual_reducao_st = float(percentual_reducao_st)

    # ICMS Próprio
    def base_icms_proprio(self):
        if self.percentual_reducao == 0:
            self.bc_icms_proprio = BaseIcmsProprio(
                self.valor_produto, self.valor_frete, self.valor_seguro,
                self.despesas_acessorias, self.valor_desconto)
            return self.bc_icms_proprio.calcular_base_icms_proprio()
        else:
            self.bc_reduzida_icms_proprio = BaseReduzidaIcmsProprio(
                self.valor_produto, self.valor_frete, self.valor_seguro,
                self.despesas_acessorias, self.valor_desconto,
                self.percentual_reducao)
            return self.bc_reduzida_icms_proprio.\
                calcular_base_reduzida_icms_proprio()

    def valor_icms_proprio(self):
        valor_icms_proprio = ValorIcmsProprio(
            self.base_icms_proprio(), self.aliq_icms_proprio).\
            calcular_valor_icms_proprio()
        return valor_icms_proprio

    # ICMS ST
    def base_icms_st(self) -> float:
        if self.percentual_reducao_st == float('0.0'):
            self.bc_icms_st = BaseIcmsST(self.base_icms_proprio(), self.mva)
            return self.bc_icms_st.calcular_base_icms_st()
        else:
            self.bc_reduzida_icms_st = BaseReduzidaIcmsST(
                self.base_icms_proprio(), self.mva, self.percentual_reducao_st)
            return self.bc_reduzida_icms_st.calcular_base_reduzida_icms_st()

    def valor_icms_st(self) -> float:
        valor_icms_st = ValorIcmsST(
            self.base_icms_st(), self.aliq_icms_st, self.valor_icms_proprio()).\
            calcular_valor_icms_st()
        return valor_icms_st



from viggofiscal.subsystem.calculo_fiscal.utils import round_abnt
from viggofiscal.subsystem.calculo_fiscal.ipi.base_ipi import BaseIpi


class Ipi50AdValorem():

    def __init__(self, valor_produto: float, valor_frete: float,
                 valor_seguro: float, despesas_acessorias: float,
                 aliquota_ipi: float):
        self.valor_produto = float(valor_produto)
        self.valor_frete = float(valor_frete)
        self.valor_seguro = float(valor_seguro)
        self.aliquota_ipi = float(aliquota_ipi)
        self.base_calculo = BaseIpi(
            valor_produto, valor_frete, valor_seguro, despesas_acessorias)

    def calcular_base_ipi(self):
        return self.base_calculo.calcular_base_ipi()

    def valor_ipi(self):
        valor_ipi = self.calcular_base_ipi() * (self.aliquota_ipi / 100)
        return round_abnt(valor_ipi, 2)



from viggocore.common.subsystem import router


class Router(router.Router):

    def __init__(self, collection, routes=[]):
        super().__init__(collection, routes)

    @property
    def routes(self):
        return [
            {
                'action': 'Listar forma_pagamento_sefaz por id',
                'method': 'GET',
                'url': self.resource_url,
                'callback': 'get',
                'bypass': True
            },
            {
                'action': 'Listar forma_pagamento_sefaz',
                'method': 'GET',
                'url': self.collection_url,
                'callback': 'list',
                'bypass': True
            }
        ]



import flask
from viggocore.common import exception, utils
from viggocore.common.subsystem import controller


class Controller(controller.Controller):

    def get_nao_resolvidos(self):
        filters = self._filters_parse()
        filters = self._filters_cleanup(filters)

        try:
            filters = self._parse_list_options(filters)
            filters = self._clean_filters(**filters)
            count = self.manager.count(**filters)
        except exception.ViggoCoreException as exc:
            return flask.Response(response=exc.message,
                                  status=exc.status)

        response = {'qtd_erros_para_resolver': count}

        return flask.Response(response=utils.to_json(response),
                              status=201,
                              mimetype="application/json")



from enum import Enum, IntEnum
from sqlalchemy import UniqueConstraint
import sqlalchemy

from viggocore.database import db
from viggocore.common.subsystem import entity


class SerieFiscalAmbiente(Enum):
    HOMOLOGACAO = 'HOMOLOGACAO'
    PRODUCAO = 'PRODUCAO'


class SerieFiscalStatus(Enum):
    ATIVO = 'ATIVO'
    INATIVO = 'INATIVO'


class SerieFiscalModelo(IntEnum):
    NFE = 55
    NFCE = 65


class SerieFiscal(entity.Entity, db.Model):

    attributes = ['domain_org_id', 'ambiente', 'modelo', 'serie',
                  'ultimo_doc', 'status', 'numero_modelo']
    attributes += entity.Entity.attributes

    domain_org_id = db.Column(
        db.CHAR(32), db.ForeignKey('domain_org.id'), nullable=False)
    ambiente = db.Column(sqlalchemy.Enum(SerieFiscalAmbiente), nullable=False)
    modelo = db.Column(sqlalchemy.Enum(SerieFiscalModelo), nullable=False)
    serie = db.Column(db.Numeric(10), nullable=False)
    ultimo_doc = db.Column(db.Numeric(15), nullable=False)
    status = db.Column(sqlalchemy.Enum(SerieFiscalStatus), nullable=False)

    __table_args__ = (
        UniqueConstraint(
            'domain_org_id', 'modelo', 'serie', 'ultimo_doc',
            name='serie_fiscal_domain_org_modelo_serie_ulitmo_doc_uk'),)

    def __init__(self, id, domain_org_id, ambiente, modelo, serie,
                 ultimo_doc, status,
                 active=True, created_at=None, created_by=None,
                 updated_at=None, updated_by=None, tag=None):
        super().__init__(id, active, created_at, created_by,
                         updated_at, updated_by, tag)
        self.domain_org_id = domain_org_id
        self.ambiente = ambiente
        self.modelo = modelo
        self.serie = serie
        self.ultimo_doc = ultimo_doc
        self.status = status

    @property
    def numero_modelo(self):
        return self.modelo.value

    @classmethod
    def individual(cls):
        return 'serie_fiscal'



from viggofiscal.subsystem.calculo_fiscal.icms.base_icms_proprio \
    import BaseIcmsProprio
from viggofiscal.subsystem.calculo_fiscal.icms.valor_icms_proprio \
    import ValorIcmsProprio


class Icms00():

    def __init__(self, valor_produto: float, valor_frete: float,
                 valor_seguro: float, despesas_acessorias: float,
                 valor_ipi: float, valor_desconto: float,
                 aliq_icms_proprio: float):
        self.valor_produto = float(valor_produto)
        self.valor_frete = float(valor_frete)
        self.valor_seguro = float(valor_seguro)
        self.despesas_acessorias = float(despesas_acessorias)
        self.valor_ipi = float(valor_ipi)
        self.valor_desconto = float(valor_desconto)
        self.aliq_icms_proprio = float(aliq_icms_proprio)
        self.base_icms = BaseIcmsProprio(
            valor_produto, valor_frete, valor_seguro, despesas_acessorias,
            valor_desconto, valor_ipi)

    def base_icms_proprio(self):
        return self.base_icms.calcular_base_icms_proprio()

    def valor_icms_proprio(self) -> float:
        return ValorIcmsProprio(
            self.base_icms_proprio(), self.aliq_icms_proprio).\
            calcular_valor_icms_proprio()



from viggofiscal.subsystem.calculo_fiscal.utils import round_abnt
from viggofiscal.subsystem.calculo_fiscal.pis.base_pis \
    import BasePis


class PisCofins01_02():

    def __init__(self, valor_produto: float, valor_frete: float,
                 valor_seguro: float, despesas_acessorias: float,
                 valor_desconto: float, aliquota_pis: float,
                 aliquota_icms: float):
        self.valor_produto = float(valor_produto)
        self.valor_frete = float(valor_frete)
        self.valor_seguro = float(valor_seguro)
        self.despesas_acessorias = float(despesas_acessorias)
        self.valor_desconto = float(valor_desconto)
        self.aliquota_pis = float(aliquota_pis)
        self.aliquota_icms = float(aliquota_icms)

    def base_pis_normal(self):
        base_pis = BasePis(
            self.valor_produto, self.valor_frete, self.valor_seguro,
            self.despesas_acessorias, self.valor_desconto, 0.0)
        return round_abnt(base_pis.calcular_base_pis(), 2)

    def valor_icms(self):
        valor_icms = (self.base_pis_normal() * (self.aliquota_icms / 100))
        return round_abnt(valor_icms, 2)

    def base_pis_menos_icms(self):
        valor_icms = self.valor_icms()
        base_pis = BasePis(
            self.valor_produto, self.valor_frete, self.valor_seguro,
            self.despesas_acessorias, self.valor_desconto, valor_icms)
        return round_abnt(base_pis.calcular_base_pis(), 2)

    def valor_pis(self):
        valor_pis = (self.base_pis_menos_icms() * (self.aliquota_pis / 100))
        return round_abnt(valor_pis, 2)



from viggofiscal.subsystem.calculo_fiscal.utils import round_abnt


class ValorIcmsST():

    def __init__(self, base_calculo_st: float, aliq_icms_st: float,
                 valor_icms_proprio: float):
        self.base_calculo_st = float(base_calculo_st)
        self.aliq_icms_st = float(aliq_icms_st)
        self.valor_icms_proprio = float(valor_icms_proprio)

    def calcular_valor_icms_st(self):
        valor_icms_st = (
            (self.base_calculo_st * (self.aliq_icms_st / 100)) -
            self.valor_icms_proprio)
        return round_abnt(valor_icms_st, 2)
